"use strict";
function crossPoint(x1, y1, x2, y2, x3, y3, x4, y4) {
    var vy1 = y2 - y1;
    var vx1 = x1 - x2;
    var c_1 = -1 * vy1 * x1 - vx1 * y1;
    var vy2 = y4 - y3;
    var vx2 = x3 - x4;
    var c_2 = -1 * vy2 * x3 - vx2 * y3;
    var c_3 = vx1 * vy2 - vx2 * vy1;
    if (c_3 === 0) {
        return [(x2 + x3) * 0.5, (y2 + y3) * 0.5];
    }
    else {
        return [(c_1 * vx2 - c_2 * vx1) / c_3, (vy1 * c_2 - vy2 * c_1) / c_3];
    }
}
function diffPoints(x1, y1, x2, y2) {
    var diffX = Math.abs(x2 - x1);
    var diffY = Math.abs(y2 - y1);
    return Math.sqrt(diffX * diffX + diffY * diffY);
}
function isCrossPointOnLine(cp, end) {
    var endsDistance = diffPoints(end[0].anchor[0], end[0].anchor[1], end[1].anchor[0], end[1].anchor[1]);
    var cp2end0Distance = diffPoints(end[0].anchor[0], end[0].anchor[1], cp[0], cp[1]);
    var cp2end1Distance = diffPoints(cp[0], cp[1], end[1].anchor[0], end[1].anchor[1]);
    return Math.abs(cp2end0Distance + cp2end1Distance - endsDistance) < 1;
}
function reverse(array) {
    var newArray = [];
    for (var i = array.length - 1; i >= 0; i--) {
        newArray.push(array[i]);
    }
    return newArray;
}
function getNextPoint(pathPoints, endIndex) {
    if (endIndex === 0) {
        return pathPoints[1];
    }
    else {
        return pathPoints[endIndex - 1];
    }
}
function decideAnchors(path) {
    var joinIndexOptions = [
        [0, 0],
        [0, path[1].pathPoints.length - 1],
        [path[0].pathPoints.length - 1, 0],
        [path[0].pathPoints.length - 1, path[1].pathPoints.length - 1],
    ];
    var cps = [];
    var diffs = [];
    var anchors = [];
    for (var i = 0; i < joinIndexOptions.length; i++) {
        anchors.push([]);
        var joinIndexPath1 = joinIndexOptions[i][0];
        var joinPointPath1 = path[0].pathPoints[joinIndexPath1];
        var joinIndexPath2 = joinIndexOptions[i][1];
        var joinPointPath2 = path[1].pathPoints[joinIndexPath2];
        var diff = diffPoints(joinPointPath1.anchor[0], joinPointPath1.anchor[1], joinPointPath2.anchor[0], joinPointPath2.anchor[1]);
        diffs.push(diff);
        var endNextPath1 = getNextPoint(path[0].pathPoints, joinIndexPath1);
        var endNextPath2 = getNextPoint(path[1].pathPoints, joinIndexPath2);
        var cp = crossPoint(joinPointPath1.anchor[0], joinPointPath1.anchor[1], endNextPath1.anchor[0], endNextPath1.anchor[1], joinPointPath2.anchor[0], joinPointPath2.anchor[1], endNextPath2.anchor[0], endNextPath2.anchor[1]);
        cps.push(cp);
        var cpOnPath1 = isCrossPointOnLine(cp, [
            joinPointPath1,
            endNextPath1,
        ]);
        var cpOnPath2 = isCrossPointOnLine(cp, [
            joinPointPath2,
            endNextPath2,
        ]);
        var slicedPath1Points = [];
        for (var j = 0; j < path[0].pathPoints.length; j++) {
            var pathPoint = path[0].pathPoints[j];
            if (!cpOnPath1 || joinIndexPath1 !== j) {
                slicedPath1Points.push(pathPoint);
            }
        }
        var slicedPath2Points = [];
        for (var j = 0; j < path[1].pathPoints.length; j++) {
            var pathPoint = path[1].pathPoints[j];
            if (!cpOnPath2 || joinIndexPath2 !== j) {
                slicedPath2Points.push(pathPoint);
            }
        }
        var reversedPath1Points = [];
        if (joinIndexPath1 !== 0) {
            reversedPath1Points = slicedPath1Points;
        }
        else {
            reversedPath1Points = reverse(slicedPath1Points);
        }
        var reversedPath2Points = [];
        if (joinIndexPath2 === 0) {
            reversedPath2Points = slicedPath2Points;
        }
        else {
            reversedPath2Points = reverse(slicedPath2Points);
        }
        for (var j = 0; j < reversedPath1Points.length; j++) {
            var point = reversedPath1Points[j];
            anchors[i].push([point.anchor[0], point.anchor[1]]);
        }
        anchors[i].push(cp);
        for (var j = 0; j < reversedPath2Points.length; j++) {
            var point = reversedPath2Points[j];
            anchors[i].push([point.anchor[0], point.anchor[1]]);
        }
    }
    var minDiff = diffs[0];
    var minDiffIndex = 0;
    for (var i = 0; i < diffs.length; i++) {
        var diff = diffs[i];
        if (diff < minDiff) {
            minDiff = diff;
            minDiffIndex = i;
        }
    }
    var points = anchors[minDiffIndex];
    return points;
}
function addLog(info, value, logText) {
    var text = logText.concat("\n", info, "\n", value);
    return text;
}
function generate() {
    var log = "";
    var selections = app.activeDocument.selection;
    if (selections === null) {
        return addLog("----------ERROR----------", "選択が無い", log);
    }
    try {
        var paths = [];
        for (var _i = 0, selections_1 = selections; _i < selections_1.length; _i++) {
            var obj = selections_1[_i];
            if (obj.typename == "PathItem") {
                paths.push(obj);
            }
        }
        if (paths.length !== 2) {
            var info_1 = "[" + new Date().toLocaleTimeString() + "] - ERROR";
            return addLog(info_1, "パスは2つ選択してください", log);
        }
        if (paths[0].pathPoints.length < 2 && paths[1].pathPoints.length < 2) {
            var info_2 = "[" + new Date().toLocaleTimeString() + "] - ERROR";
            return addLog(info_2, "パスが想定外です", log);
        }
        var pathPointAnchors = decideAnchors([paths[0], paths[1]]);
        var newPath = paths[0].duplicate();
        newPath.stroked = true;
        newPath.setEntirePath(pathPointAnchors);
    }
    catch (error) {
        var info_3 = "[" + new Date().toLocaleTimeString() + "] - ERROR";
        return addLog(info_3, error, log);
    }
    var info = "[" + new Date().toLocaleTimeString() + "] - SUCCESS";
    return addLog(info, "パスは作成されました", log);
}
